# comancheConsole --
#
# This is the main Comanche window backbone, it controls different views and
# namespaces to control different machines at once.

class comancheConsole {
    
    # List containing available views. Views are tree like + right pane 
    # widgets. There canbe more than one view, although in the current 
    # incarnation only one is considered so far.
    
    variable views

    # List of namespaces the console is currently connected to

    variable namespaces

    # For processing certain events, we need to know which view is in use
    # at a given moment

    variable currentView

    # Hack until we support proper remote configuration
    # This is done to intercept calls to the main node, which is not answered
    # by plugins (TO-DO: one way make the manager plugin handle this)

    variable rootNode

    # Diverse XML structures to read/answer requests

    variable queryChildrenData
    variable queryChildrenParentNode
    variable xuiDocumentNode
    variable xuiDocumentStructure
    variable xuiDocumentLabel
    variable xuiDocumentExtraInfo
    variable xuiAnswerNode
    variable xuiAnswerStructure
    variable xuiAnswerLabel
    variable xuiAnswerPP
    variable xuiDeleteNode
    # Contains mapping view, viewnode -> namespace, namespaceNode
    
    variable View2NsArray

    # Contains mapping namespace, namespaceNode -> view, viewnode  

    variable Ns2ViewArray

    # Contain namespace, node of a node that was copied into the clipboard

    variable clipboardNamespace
    variable clipboardNode

    constructor {} {
	
	# TO-DO. the following can be simplified by creating generic 
	# containers, etc. (xuiData)
	
	set queryChildrenParentNode [xuiNode ::#auto]
	$queryChildrenParentNode setName parentNode
	
	set xuiDocumentStructure [xuiStructure ::#auto]
	$xuiDocumentStructure setXuiClass structure
        set xuiDocumentNode [xuiNode ::#auto]
        $xuiDocumentNode setXuiClass node
        $xuiDocumentNode setName node
        set xuiDocumentLabel [xuiLabel ::#auto]
        $xuiDocumentLabel setXuiClass label
        $xuiDocumentLabel setName docType
	$xuiDocumentLabel setValue propertyPages
	set xuiDocumentExtraInfo [xuiLabel ::#auto]
        $xuiDocumentExtraInfo setXuiClass label
        $xuiDocumentExtraInfo setName extraInfo
	$xuiDocumentExtraInfo setValue {}
	$xuiDocumentStructure addComponent $xuiDocumentNode
        $xuiDocumentStructure addComponent $xuiDocumentLabel
	$xuiDocumentStructure addComponent $xuiDocumentExtraInfo
	
	
	set xuiAnswerStructure [xuiStructure ::#auto]
	$xuiAnswerStructure setXuiClass structure
        set xuiAnswerNode [xuiNode ::#auto]
        $xuiAnswerNode setXuiClass node
        $xuiAnswerNode setName node
        set xuiAnswerLabel [xuiLabel ::#auto]
        $xuiAnswerLabel setXuiClass label
        $xuiAnswerLabel setName docType
	$xuiAnswerLabel setValue propertyPages
	set xuiAnswerPP [xuiStructure ::#auto]
	$xuiAnswerPP setName propertyPages
	$xuiAnswerStructure addComponent $xuiAnswerNode
        $xuiAnswerStructure addComponent $xuiAnswerLabel
        $xuiAnswerStructure addComponent $xuiAnswerPP

        set xuiDeleteNode [xuiNode ::#auto]
	
	set descmenu {
	    "&Comanche" all file 0 {
		{command "E&xit" {} "Exit Comanche" {} -command exit}
	    }
	    "&Help" all help 0 {
		{
		    command "&Help" {} "Help" {} -command {
			global tcl_platform
			if {$tcl_platform(platform) == "windows"} {
			    exec explorer http://www.comanche.org &
			} else {
			    exec netscape http://www.comanche.org &
		    }   
		}   
	    }
	    {command "&About" {} "About Comanche" {} -command {
		MessageDlg .d -message "Comanche $::comanche::version \n\
			January 2000\n\n\
			http://www.comanche.org\n\n\
			Sponsored by Covalent Technologies\n\
			\n\
			Brought to you by The Comanche Team:\n\
			Carmen Carmack, Craig Khotz, Daniel Lopez,\n\
			Ryan Morgan, Adam Qualset \n\n\
			Many thanks to :\n\
			Lars Dittman, COM Center http://www.com.dtu.dk\n\
			Jorge Chavez, ESI http://www.esi.us.es\n" -type ok  
	    }
	}
    }   
}
    set shell [MainFrame .main \
	    -textvariable global_help -separator both  -menu $descmenu]
    $shell addindicator -textvariable global_help
    set tb1 [$shell addtoolbar]
    set bbox [ButtonBox $tb1.bbox1 -spacing 0 -padx 3 -pady 3]
    pack $bbox -side left -anchor w      
    set sep [Separator $tb1.sep1 -orient vertical]
    pack $sep -side left -fill y -padx 4 -anchor w       
    set bbox [ButtonBox $tb1.bbox2 -spacing 0 -padx 3 -pady 3]
    pack $bbox -side left -anchor w      
    $bbox add -image help \
	    -highlightthickness 0 -takefocus 0 -relief link \
	    -borderwidth 1	-padx 1 -pady 1 \
	    -helptext "Help"\
	    -command {    exec netscape http://www.comanche.org & }
    $bbox add -image about \
	    -highlightthickness 0 -takefocus 0 -relief link \
	    -borderwidth 1	-padx 1 -pady 1 \
	    -helptext "About Comanche"\
	    -command {
	MessageDlg .d -message "Comanche $::comanche::version \n\
			January 2000\n\n\
			http://www.comanche.org\n\n\
			Sponsored by Covalent Technologies\n\
			\n\
			Brought to you by The Comanche Team:\n\
			Carmen Carmack, Craig Khotz, Daniel Lopez,\n\
			Ryan Morgan, Adam Qualset \n\n\
			Many thanks to :\n\
			Lars Dittman, COM Center http://www.com.dtu.dk\n\
			Jorge Chavez, ESI http://www.esi.us.es\n" -type ok  
    } 
    set bbox [ButtonBox $tb1.bbox3 -spacing 0 -padx 3 -pady 3]
    pack $bbox -side left -anchor w      
    $bbox add -image back \
	    -highlightthickness 0 -takefocus 0 -relief link \
	    -borderwidth 1 -padx 1 -pady 1 \
	    -helptext "Go back one level"\
	    -command  "$this processSignal levelUp"
    $bbox add -image refresh \
	    -highlightthickness 0 -takefocus 0 -relief link \
	    -borderwidth 1 -padx 1 -pady 1 \
	    -helptext "Refresh display"
    set sep [Separator $tb1.sep3 -orient vertical]
    pack $sep -side left -fill y -padx 4 -anchor w       
    set bbox [ButtonBox $tb1.bbox4 -spacing 0 -padx 3 -pady 3]
    $bbox add -image [Bitmap::get cut] \
	    -highlightthickness 0 -takefocus 0 -relief link \
	    -borderwidth 1 -padx 1 -pady 1 \
	    -helptext "Cut selection"\
	    -command { \
	    MessageDlg .d -message "Cut a node not implemented"  -type ok \
	}
    $bbox add -image [Bitmap::get copy] \
	    -highlightthickness 0 -takefocus 0 -relief \
	    link -borderwidth 1 -padx 1 -pady 1 \
	    -helptext "Copy selection"\
	    -command { \
	    MessageDlg .d -message "Copy a node not implemented"  -type ok \
	}
    $bbox add -image [Bitmap::get paste] \
	    -highlightthickness 0 -takefocus 0 \
		-relief link -borderwidth 1 -padx 1 -pady 1 \
		-helptext "Paste selection"\
	        -command { \
		MessageDlg .d -message "Paste a node not implemented" -type ok\
	    }
    pack $bbox -side left -anchor w      
    set sep [Separator $tb1.sep2 -orient vertical]
    pack $sep -side left -fill y -padx 4 -anchor w       
    set bbox [ButtonBox $tb1.bbox5 -spacing 0 -padx 3 -pady 3]
    $bbox add -image locale \
	    -highlightthickness 0 -takefocus 0 \
		-relief link -borderwidth 1 -padx 1 -pady 1 \
		-helptext "Change default language"\
	        -command { \
		::mesg::changeLanguageDialog
	    }
    pack $bbox -side left -anchor w      
    set view [view ::#auto [$shell getframe] $this]
    addView $view
    set currentView $view
    pack $shell -fill both -expand true
    guiBuilder ::kk
    propertyPagesDialog ll ::kk
    wizardDialog ww ::kk
    commandResult cc ::kk
}
    
    method addView { newView } { lappend views $newView }
    method newNamespace { namespace }
    method populateRequestNotify { viewNode view }
    method nodeAddedNotify { xuiData caller }
    method nodeRemovedNotify { xuiData caller }
    method nodeSelectedNotify { viewNode view }
    method nodeModifiedNotify { xuiData caller }
    method showMenuRequest { view node}
    method _addNode { view parentNode namespace xuiNode }
    method _removeNode { view node namespace xuiNode}
    method _configureNode { view node namespace xuiNode }
    method deleteNodeRequest { namespace node }
    method processSignal { signal args}
    method showPropertyPages { namespace node }
    method copyNode {}
    method pasteNode
    method showWizard {type namespace node}    
    method showCommandResult {namespace node command}
    method refreshRightPane
}

body comancheConsole::refreshRightPane {} {

    # By default, will select current node

    processSignal selectNode
}

body comancheConsole::processSignal {signal args} {
    array set options { \
	    -namespaceNode {} \
	    -namespace {} \
	    -view {} \
	    -viewNode {} }
    array set options $args
    if ![ string length $options(-view)] {
	catch [list set options(-view) $currentView]
    }
    if ![ string length $options(-viewNode)] {
	catch list [set options(-viewNode) [$currentView getSelectedNode]]
    }
    catch [list foreach {namespace node}\
	    $View2NsArray([list $options(-view) $options(-viewNode)]) break]
    if [string length $options(-namespaceNode)] {
        catch [list set node $options(-namespaceNode)]
    } 
    switch $signal {
	copyNode {
	} levelUp {
	    catch " \
	    $options(-view) selectNode \
	    [$options(-view) getParentNode $options(-viewNode)] "
	} selectViewNode {
	} propertyPages {
	    foreach {namespace node} \
		    $View2NsArray([list $options(-view) $options(-viewNode)])\
		    break	
	    showPropertyPages $namespace $node
	} addNewNode {
	    showWizard add $namespace $node
	} selectNode {
	    foreach pair $Ns2ViewArray([list $namespace $node]) {
		foreach {view viewNode} $pair break
		if { $options(-view) == $view } {
		    $options(-view) selectNode $viewNode
		}
	    }
	} default {
	   # All the other not known commands will be send as-is to the plugIn
	   showCommandResult $namespace $node $signal
	}
    }
}

body comancheConsole::showMenuRequest { view node } {
    foreach {namespace nodeId} $View2NsArray([list $view $node]) break
    set mymenu [menu .[unique::newId] -tearoff 0]
    $mymenu add command -label "Add a new node" \
	    -command "$this showWizard add $namespace $nodeId"
    $mymenu add command -label "Delete this node" \
	    -command "$this deleteNodeRequest $namespace $nodeId"
    $mymenu add separator
    $mymenu add command -label "Cut" -state disabled
    $mymenu add command -label "Copy"
    $mymenu add command -label "Paste"
    $mymenu add separator
    $mymenu add command -label "Configure"\
	    -command " $this showPropertyPages $namespace $nodeId"
    tk_popup $mymenu [winfo pointerx .] [winfo pointery .]
}

body comancheConsole::deleteNodeRequest {namespace nodeId} {
    if ![string compare $nodeId $rootNode] { return }
    $xuiDeleteNode setId $nodeId
    $namespace deleteNodeRequest $xuiDeleteNode $this 
}

body comancheConsole::showPropertyPages { namespace nodeId } {
    if ![string compare $nodeId $rootNode] { return }
    $xuiDocumentLabel setValue propertyPages
    $xuiDocumentExtraInfo setValue ""
    $xuiDocumentNode setId $nodeId
    set answer [$namespace requestXuiDocument $xuiDocumentStructure $this]
    switch  [ll show $answer] {
	OK {
	    $xuiAnswerLabel setValue propertyPages
	    $xuiAnswerNode setId $nodeId
	    $answer copyClone $xuiAnswerPP
	    $xuiAnswerPP setName propertyPages
	    set result \
		    [$namespace answerXuiDocument $xuiAnswerStructure $this]
	    $xuiAnswerPP clear
	    $xuiAnswerPP clear
	    refreshRightPane
	} CANCEL {
	}
    }
}


body comancheConsole::showWizard { type namespace nodeId } {
    if ![string compare $nodeId $rootNode] { return }
    $xuiDocumentLabel setValue wizardAdd
    $xuiDocumentExtraInfo setValue ""
    $xuiDocumentNode setId $nodeId
    set answer [$namespace requestXuiDocument $xuiDocumentStructure $this]
    switch  [ww show $answer] {
	OK {
	    $xuiAnswerLabel setValue wizardAdd
	    $xuiAnswerNode setId $nodeId
	    $answer copyClone $xuiAnswerPP
	    $xuiAnswerPP setName propertyPages
	    set result \
		    [$namespace answerXuiDocument $xuiAnswerStructure $this] 
	} CANCEL {
	}
    }
}

body comancheConsole::showCommandResult { namespace nodeId commandName} {
    $xuiDocumentLabel setValue command
    $xuiDocumentExtraInfo setValue $commandName
    $xuiDocumentNode setId $nodeId
    set answer [$namespace requestXuiDocument $xuiDocumentStructure $this]
    switch  [cc show $answer] {
	OK {
	} CANCEL {
	}
    }
}

body comancheConsole::copyNode { } {
    foreach {clipboardNamespace  clipboardNode} \
	    $View2NsArray([list $currentView [$currentView getSelectedNode]])\
	    break
}

body comancheConsole::pasteNode { } {
}


body comancheConsole::populateRequestNotify { parentNode view } {
    foreach {namespace nodeId} $View2NsArray([list $view $parentNode]) break
    $queryChildrenParentNode setId $nodeId
    set xuiData [$namespace getChildren $queryChildrenParentNode $this]
    foreach child [$xuiData getChildren] {
	_addNode $view $parentNode $namespace $child
    }
}

body comancheConsole::nodeSelectedNotify { node view } {
    foreach {namespace nodeId} $View2NsArray([list $view $node]) break
    if ![string compare $nodeId $rootNode] {
    $view loadHTML "<center><img src=\"logo\"  /></center>\
            [mesg::get comancheintro]"
	return
    }
    $xuiDocumentNode setId $nodeId
    $xuiDocumentLabel setValue description
    set answer [$namespace requestXuiDocument $xuiDocumentStructure $this]
    $view loadHTML $answer
}

body comancheConsole::_addNode { view parentNode namespace xuiNode }  {
    set type leaf
    if [$xuiNode doYouBelongTo container] {
           set type container
    }
    set newViewNode [$view addNode $parentNode \
	    -openImage [$xuiNode getOpenIcon] \
	    -closedImage [$xuiNode getClosedIcon] \
	    -text [$xuiNode getLabel] \
	    -type $type ]
    set View2NsArray([list $view $newViewNode]) \
	    [list $namespace [$xuiNode getId]]
    lappend Ns2ViewArray([list $namespace [$xuiNode getId]]) \
	    [list $view $newViewNode]
}

body comancheConsole::_removeNode { view node namespace xuiNode }  {
    $view deleteNode $node 
    unset View2NsArray([list $view $node]) 
    unset Ns2ViewArray([list $namespace [$xuiNode getId]])
}

body comancheConsole::newNamespace { namespace } {
   lappend namespaces $namespace
    set rootNode [[set node [$namespace getRootNode]] getId]
    foreach view $views {
	$this _addNode $view root $namespace $node
    }
}

body comancheConsole::nodeAddedNotify { xuiData caller } {
    set parentNodeId [[$xuiData getComponentByName parentNode] getId]
    foreach pair $Ns2ViewArray([list $caller $parentNodeId]) {
	foreach {view viewNode} $pair break
	$this _addNode $view $viewNode $caller \
		[$xuiData getComponentByName addedNode]
    }
}

body comancheConsole::nodeRemovedNotify { xuiData caller } {
    set nodeId [[$xuiData getComponentByName node] getId]
    foreach pair $Ns2ViewArray([list $caller $nodeId]) {
	foreach {view viewNode} $pair break
	$this _removeNode $view $viewNode $caller \
		[$xuiData getComponentByName node]
    }
}


body comancheConsole::nodeModifiedNotify { xuiData caller } {
    set nodeId [$xuiData getId]
    foreach pair $Ns2ViewArray([list $caller $nodeId]) {
	foreach {view viewNode} $pair break
	$this _configureNode $view $viewNode $caller $xuiData
    }
}


body comancheConsole::_configureNode { view node namespace xuiNode }  {
    $view configureNode $node -openImage [$xuiNode getOpenIcon]\
	    -closedImage [$xuiNode getClosedIcon]\
	    -text [$xuiNode getLabel ] \
	    -type container 
}
